!+
! Module: make_backcorr_mod
!
! Authors:  Didier Barret - CESR
!          
! Description: Makes the actual correction once arfgen has been called
! 
! Comments: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module make_backcorr_mod

  use dal
  use cal
  use param  
  use metatask_mod
  use errorHandling
  use caloaldefs

!!$  use param  
!!$  use dal
!!$  use cal
!!$  use errorHandling

  implicit none 

  private 

  public make_backcorr


contains 

  !---------------------------------------------------------------------
  subroutine make_backcorr
  !---------------------------------------------------------------------

 
! The input parameters of the task:
! infile : input spectrum file for backcorr correction
! outfile : output spectrum file produced by Backcorr
! arfbkgfile : arf file for the background region
! arfsrcfile : arf file for the source region

  character (len=4096)            :: infile, outfile
  character (len=4096)            :: arfbkgfile, arfsrcfile

! A few datasets to access both the arf files (SpecResp*) and spectrum files (SpBkg*)
! The energy bins of the arf files (Elow and Ehigh)

  type(DataSetT) :: ArfDataSet,SpDataSet
  type(TableT) :: tableSet
  type(ColumnT) :: SpecRespSrc, SpecRespBkg, SpBkg, SpBkg_Channel
  type(ColumnT) ::  ELow, Ehigh

! Now some arrays representing the actual values of the arf and energy bins
! Channel : Channel numbers  of the input spectrum (nbChannels)
! arf_src and arf_bkg : arf arrays for the source and background 
! Emin_sp, Emax_sp : Energy bins of the spectrum to correct
! Emin_arf, Emax_arf : Energy bins of the arf curves

  integer(kind=int16),dimension(:),pointer :: Channel
  integer :: nb_channels
  real(kind=single),dimension(:),pointer :: arf_src,arf_bkg
  real(double), dimension(:), pointer   :: Emin_sp, Emax_sp
  real(kind=single),dimension(:),pointer :: Emin_arf, Emax_arf


! The actual array on which correction are performed

  integer(kind=int32), Dimension(:), pointer :: SpToCor

! A few local and useless variables

  integer :: i,j,iarfmin,iarfmax
  real (kind=single):: mean_arf
  character(len=80) :: errText
 
! If you have reached this point, then the arfgen calls have been succesfully
! completed
    
!  call message(AppMsg,Sparse,"Make_Backcorr... running")

!=======================================================================
! Get ALL INPUT PARAMETERS for the task
!=======================================================================
 
  infile=stringParameter("inset")  
  outfile=stringParameter("outset")  
  arfsrcfile=stringParameter("arfsrcset")  
  arfbkgfile=stringParameter("arfbkgset")  

! To call CAL_getEbounds you need first the setup the CAL state

  call setcalstate(trim(infile))


! Now reading the arf file for the source

  ArfDataSet = dataSet(arfsrcfile, read)
  tableSet = table(ArfDataSet, "SPECRESP")
  SpecRespSrc = column(tableSet,"SPECRESP", read)
  arf_src => real32Data(SpecRespSrc)
  Elow = column(tableSet,"ENERG_LO", read)
  Ehigh = column(tableSet,"ENERG_HI", read)


! Emin_arf and Emax_arf are assumed to be the same for the arf in
! the source and background regions

  Emin_arf => real32data(Elow)
  Emax_arf => real32data(Ehigh)
  
! Now reading the arf file for the background

  ArfDataSet = dataSet(arfbkgfile, read)
  tableSet = table(ArfDataSet, "SPECRESP")
  SpecRespBkg = column(tableSet,"SPECRESP", read)
  arf_bkg => real32Data(SpecRespBkg)

! Now writing the output background spectrum
! The output spectrum will have exactly the same header as the input
! spectrum. Its history keyworks will reflect the present call to
! backcorr

  call CopyDataSet(infile,outfile)

! Now starts reading the Count spectrum (in fact that is the same
! as the input spectrum)

  SpDataSet = dataSet(outfile,modify)
  tableSet = table(SpDataSet, "SPECTRUM")
  SpBkg = column(tableSet,"COUNTS", read)
  SpBkg_Channel=Column(tableset , ColumnNumber(tableset,"CHANNEL"),Read)
  Channel => int16Data(SpBkg_Channel)
  SpToCor => int32data(SpBkg)
  nb_channels=size(Channel)


  if(sum(SpToCor) <= 0) call warning('Backcorr',&
  'The input spectrum for Backcorr has a negative or null total number of counts!')

  if(maxval(SpToCor) < 0) call warning('Backcorr',&
  'The input spectrum for Backcorr has negative number of counts everywhere!')

! Get Emin_sp, Emax_sp : Energy bins for the input spectrum

  call CAL_getEbounds(Channel, Emin_sp, Emax_sp)

! The energy bounds of the input spectrum are in eV (whereas the channel
! boundaries of arfgen) are in keV

  Emin_sp=Emin_sp/1000.0
  Emax_sp=Emax_sp/1000.0

! The heart of Make_Backcorr:
! Basically, for each energy bin defined by Emin_sp and Emax_sp 
! of the input spectrum it  determines the corresponding 
! energy bins of the corresponding arf. If the energy range
! defined by Emin_sp and Emax_sp contains several arf bins, then a 
! simple average of the arf ratios over the range is performed.
! Now the input spectrum is in counts (int32); that must be multiplied 
! by the arf ratio which may  may be in many cases close to 1 or so. 
! Since the output spectrum must also be in counts (int32), I used the 
! nearest integer to convert from real to int32

  theiloop: do i = 0,nb_channels-1

    iarfmin=-1
    iarfmax=-1
    thejloop: do j = 0,size(emin_arf)-1
      If (Emax_sp(i) < emin_arf(j)) cycle theiloop
      If (Emin_sp(i) > emax_arf(j)) cycle thejloop
      If (Emin_sp(i) >= emin_arf(j) .and. Emin_sp(i) < emax_arf(j))  iarfmin=j
      If (Emax_sp(i) > emin_arf(j) .and. Emax_sp(i) <= emax_arf(j))  iarfmax=j
     enddo thejloop
    if (iarfmin ==-1 .or. iarfmax ==-1) then
       write(unit=errText,fmt=*)'Energy bin : ',i,' = ',Emin_sp(i),Emax_sp(i),&
                 ' keV not contained in arf curve ==> ARF ratio set to 1'
       call warning('Backcorr',errText)
       iarfmin=0
       iarfmax=0
    endif
    mean_arf=sum(arf_src(iarfmin:iarfmax))/sum(arf_bkg(iarfmin:iarfmax))
    mean_arf=mean_arf/(iarfmax-iarfmin+1)
    SpToCor(i)=nint(mean_arf*SpToCor(i))
  enddo theiloop


  if(maxval(SpToCor) < 0) call warning('Backcorr',&
  'The Backcorr corrected spectrum is negative everywhere !')

  if(sum(SpToCor) <= 0) call warning('Backcorr',&
  'The Backcorr corrected spectrum has a negative or null total number of counts !')

  call release(SpDataset)
  call release(ArfDataset)


  end subroutine make_backcorr


  ! Read miscellaneous information from the supplied spectrum file
  ! and set up the CAL
  !
  !---------------------------------------------------------------------
  subroutine setcalstate(inFile)
  !---------------------------------------------------------------------
    character (len=*), intent(in) :: inFile

    ! Local variables

    character (len=5) :: instrument
    type (DataSetT) :: set
    character(len=*),parameter :: cifName = ''


    set=dataSet(inFile,Read)

    ! Read instrument and work out the instrument code

    instrument=stringAttribute(set,"INSTRUME")

    ! Set CAL state

    select case (Instrument)
      case ('EMOS1')
        call CAL_setState(EMOS1)
      case ('EMOS2')
        call CAL_setState(EMOS2)
      case ('EPN')
        call CAL_setState(EPN)
      case default
        call error('Wrong instrument name', &
             'Should be either EMOS1, EMOS2, EPN')
    end select

    call release(set)

  end subroutine setcalstate
    
end module make_backcorr_mod
